//defining connections
#define HALL 2  //hall effect sensor
#define POT_SPEED A1  //potentiometer controlling the speed
#define POT_SWING A2  //potentiometer controlling the amplitude
#define DC_STBY 9 //standby pin of the motor controller
#define DC_AIN1 8 //in1 pin of the motor controller
#define DC_AIN2 7 //in2 pin of the motor controller
#define DC_PWM  6 //pwm pin of the motor controller
#define STEP_EN   A0  //enable pin of the stepper controller
#define STEP_MS1  13  //ms1 pin of the stepper controller
#define STEP_MS2  12  //ms2 pin of the stepper controller
#define STEP_STEP 11  //step pin of the stepper controller
#define STEP_DIR  10  //direction pin of the stepper controller

/*-------------------------------------------------------------------*/

#define MICROSTEPPING 1 //step divider, possible values: 1, 2, 4, 8
#define REVERT_STEPPER false  //stepper diection can be reverted
#define DC_ON HIGH   //state of the stby pin when the motor is on
#define STEP_ON LOW  //steta of the en pin when the stepper is on

/*-------------------------------------------------------------------*/

void setup(void) {
  dc_begin(DC_STBY, DC_AIN1, DC_AIN2, DC_PWM);  //initialize the dc motor driver
  step_begin(STEP_EN, STEP_MS1, STEP_MS2, STEP_STEP, STEP_DIR); //initialize the stepper driver
  pinMode(HALL, INPUT); //set hall effect pin as input
  attachInterrupt(digitalPinToInterrupt(HALL), hall_change_dc_dir, CHANGE); //attach interrupt to the hall effect sensor
  dc_SetDirection(DC_AIN1, DC_AIN2, HIGH);  //forward direction
  dc_SetSpeed(DC_PWM, analogRead(POT_SPEED)); //start the dc motor
}

/*-------------------------------------------------------------------*/

//global variable
int previous_position = 0;  //used to remember last position of the stepper
int previous_speed = 0; //used to remember last speed of the dc motor
int current_steps = 0;  //used to remember the current position of the stepper

/*-------------------------------------------------------------------*/

void loop(void) {
  //set stepper position (swing amplitude)
  int current_position = analogRead(POT_SWING) - current_steps; //read potentiometer value, extract step count
  if(current_position != previous_position) { //compare it to its previous state
    current_steps += current_position * 100;  //update step count
    stp_TravelTo(STEP_DIR, STEP_STEP, current_position); //change stepper position if not equal
    previous_position = current_position; //save current state
  }

  //set dc motor speed
  int current_speed = analogRead(POT_SPEED); //read potentiometer value
  if(current_speed != previous_speed) { //compare it to its previous state
    dc_SetSpeed(DC_PWM, current_speed); //change dc motor speed if not equal
    previous_speed = current_speed; //save current state
  }
}

/*-------------------------------------------------------------------*/

//interrupt service routine for Hall effect interrupt
void hall_change_dc_dir(void) {
  dc_SetDirection(DC_AIN1, DC_AIN2, digitalRead(HALL)); //change motor direction
  return;
}

/*-------------------------------------------------------------------*/

//initialize the dc motor
void dc_begin(unsigned int STBY, unsigned int AIN1, unsigned int AIN2, unsigned int PWM) {
  //set pins as output
  pinMode(AIN1, OUTPUT);
  pinMode(AIN2, OUTPUT);
  pinMode(STBY, OUTPUT);
  pinMode(PWM, OUTPUT);

  //stop the motor
  digitalWrite(AIN1, LOW);
  digitalWrite(AIN2, LOW);
  digitalWrite(PWM, HIGH);

  //activate the motor
  digitalWrite(STBY, DC_ON);
  return;
}

//set the direction of the dc motor
void dc_SetDirection(unsigned int AIN1,unsigned int AIN2, bool forward) {
  if(forward) {  //forward
    digitalWrite(AIN1, HIGH); //set forwad pin configuration
    digitalWrite(AIN2, LOW);
  }
  else {  //reverse
    digitalWrite(AIN1, LOW); //set reverse pin configuration
    digitalWrite(AIN2, HIGH);
  }
  return;
}

//set the speed of the dc motor
void dc_SetSpeed(unsigned int PWM, int spd) {
  analogWrite(PWM, map(spd, 0, 1023, 0, 255)); //set duty cycle for pwm signal
  return;
}

/*-------------------------------------------------------------------*/

//initialize the stepper motor
void step_begin(unsigned int EN, unsigned int MS1, unsigned int MS2, unsigned int STEP, unsigned int DIR) {
  //set pins as output
  pinMode(EN, OUTPUT);
  pinMode(MS1, OUTPUT);
  pinMode(MS2, OUTPUT);
  pinMode(STEP, OUTPUT);
  pinMode(DIR, OUTPUT);
  digitalWrite(STEP, HIGH); //starting state

  //set microstepping
  switch(MICROSTEPPING) {
    case 1: digitalWrite(MS1, LOW); digitalWrite(MS2, LOW); break;  //full step
    case 2: digitalWrite(MS1, HIGH); digitalWrite(MS2, LOW); break; //half step
    case 4: digitalWrite(MS1, LOW); digitalWrite(MS2, HIGH); break; //quarter step
    case 8: digitalWrite(MS1, HIGH); digitalWrite(MS2, HIGH); break;  //eight step
    default: digitalWrite(MS1, LOW); digitalWrite(MS2, LOW); break; //default: full step
  }

  //set direction
  digitalWrite(DIR, REVERT_STEPPER);

  //enable stepper
  digitalWrite(EN, STEP_ON);
  return;
}

//change stepper position
void stp_TravelTo(unsigned int DIR, unsigned int STEP, int pos) {
  if(pos>0) { //positive direction
    digitalWrite(DIR, REVERT_STEPPER);  //keep direction
  }
  else {
    digitalWrite(DIR, !REVERT_STEPPER); //change direction
    pos = -pos; //calculate module
  }

  //make steps
  for(int i=0; i<pos; i++) {
    digitalWrite(STEP, LOW);
    digitalWrite(STEP, HIGH);
  }
  return;
}
